import nacl.public
import nacl.pwhash
import nacl.signing

from v25.messaging.encoding import Encoding


class Subject:
    def __init__(self, vkey: bytes):
        self.vkey = nacl.signing.VerifyKey(vkey)
        self.pkey: nacl.public.PublicKey = self.vkey.to_curve25519_public_key()

    def __eq__(self, other):
        if not isinstance(other, Subject):
            return False
        return self.vkey == other.vkey

    def __hash__(self):
        return hash(self.vkey.encode())

    def dumps(self) -> str:
        return Encoding.encode(self.vkey)

    @classmethod
    def loads(cls, s: str):
        return Subject(Encoding.decode(s))


class PrivateSubject(Subject):
    def __init__(self, pwd: str):
        salt = nacl.pwhash.argon2id.SALTBYTES * b'\0'
        self.seed = nacl.pwhash.argon2id.kdf(
            nacl.public.PrivateKey.SEED_SIZE, pwd.encode(), salt,

            nacl.pwhash.OPSLIMIT_INTERACTIVE,
            nacl.pwhash.MEMLIMIT_INTERACTIVE)
        self.skey = nacl.signing.SigningKey(self.seed)
        self.ekey: nacl.public.PrivateKey = self.skey.to_curve25519_private_key()
        super().__init__(bytes(self.skey.verify_key))
