from abc import ABC
from typing import Tuple, Optional, List, Iterable

from v25.messaging.message import Message
from v25.messaging.subject import Subject


class AbstractStorage(ABC):
    def check(self, subject: Subject) -> dict:
        raise NotImplementedError

    def push(self, m: Message) -> None:
        raise NotImplementedError

    def edit(self, old: Message, new: Message):
        raise NotImplementedError

    def delete(self, m: Message):
        raise NotImplementedError

    def pull(self, pair: Tuple[Subject, Subject], params: Optional[dict] = None) -> Iterable[Message]:
        raise NotImplementedError

    def flags(self, m: Message, flags: str):
        raise NotImplementedError
